use x11::xinerama;

use crate::errors::Result;
use crate::xlib::Display;

pub struct ScreensInfo {
  inner: *mut xinerama::XineramaScreenInfo,
  heads: isize,
}

impl ScreensInfo {
  pub fn query(display: &Display) -> Result<Self> {
    let mut heads = 0;
    let inner = unsafe {
      xinerama::XineramaQueryScreens(display.inner, &mut heads as *mut _)
    };
    // TODO: check for error
    Ok(ScreensInfo {
      inner,
      heads: heads as isize,
    })
  }

  pub fn iter(&self) -> ScreenInfoIter {
    ScreenInfoIter(self, 0)
  }
}

impl Drop for ScreensInfo {
  fn drop(&mut self) {
    unsafe { x11::xlib::XFree(self.inner as *mut _) };
  }
}

pub struct ScreenInfoIter<'a>(&'a ScreensInfo, isize);

pub struct ScreenInfo {
  pub x: i16,
  pub y: i16,
  pub width: i16,
  pub height: i16,
}

impl<'a> Iterator for ScreenInfoIter<'a> {
  type Item = (i32, ScreenInfo);
  fn next(&mut self) -> Option<Self::Item> {
    if self.1 >= self.0.heads {
      return None;
    }

    let screen = unsafe { self.0.inner.offset(self.1) };
    self.1 += 1;

    unsafe {
      Some((
        (*screen).screen_number as i32,
        ScreenInfo {
          x: (*screen).x_org as i16,
          y: (*screen).y_org as i16,
          width: (*screen).width as i16,
          height: (*screen).height as i16,
        },
      ))
    }
  }
}
