use x11::xlib;

use crate::errors::Result;
use crate::rect::Rectangle;

use super::display::GetDisplay;
use super::image::Image;

/// Anything that's drawable
pub trait Drawable: GetDisplay {
  /// Get drawable handle
  fn as_drawable(&self) -> xlib::Drawable;

  /// Capture a snapshot of this drawable, clipped by rect.
  fn get_image(&self, rect: Rectangle) -> Result<Image> {
    let image = unsafe {
      xlib::XGetImage(
        self.get_display(),
        self.as_drawable(),
        rect.x as i32,
        rect.y as i32,
        rect.width,
        rect.height,
        0xffffffff,
        xlib::ZPixmap,
      )
    };
    Ok(Image { inner: image })
  }
}
