//! The core of x11

mod atom;
mod colormap;
mod cursor;
mod display;
mod drawable;
mod event;
mod image;
mod pixmap;
mod screen;
mod visual;
mod window;

use x11::xlib;

pub use self::atom::Atom;
pub use self::colormap::Colormap;
pub use self::cursor::Cursor;
pub use self::display::{Display, GetDisplay};
pub use self::drawable::Drawable;
pub use self::event::{Event, EventKind};
pub use self::image::{Image, ImageByteOrder, PixBuffer};
pub use self::pixmap::Pixmap;
pub use self::visual::{Visual, VisualInfo};
pub use self::window::{
  EventMask, SetWindowAttributes, Window, WindowAttributes,
};

#[derive(Debug)]
pub enum Inherit<T> {
  Value(T),
  CopyFromParent,
}

impl<T> Default for Inherit<T> {
  fn default() -> Self {
    Inherit::CopyFromParent
  }
}

pub trait IntoInt {
  fn into_int(&self) -> u64;
}

impl<T: IntoInt> IntoInt for Inherit<T> {
  fn into_int(&self) -> u64 {
    match self {
      Inherit::Value(t) => t.into_int(),
      Inherit::CopyFromParent => xlib::CopyFromParent as u64,
    }
  }
}

#[derive(Debug)]
pub enum Noneable<T> {
  Value(T),
  None,
}

impl<T> Default for Noneable<T> {
  fn default() -> Self {
    Noneable::None
  }
}

impl<T: IntoInt> IntoInt for Noneable<T> {
  fn into_int(&self) -> u64 {
    match self {
      Noneable::Value(t) => t.into_int(),
      Noneable::None => 0,
    }
  }
}
