use x11::xlib;

use crate::xlib::{Display, IntoInt};

/// Pixmaps are off-screen resources that are used for various operations, for
/// example, defining cursors as tiling patterns or as the source for certain
/// raster operations
///
/// Most graphics requests can operate either on a window or on a pixmap. A
/// bitmap is a single bit-plane pixmap. Pixmaps can only be used on the screen
/// on which they were created.
#[derive(Derivative)]
#[derivative(Debug)]
pub struct Pixmap<'a> {
  #[derivative(Debug = "ignore")]
  pub(crate) display: &'a Display,
  pub(crate) inner: xlib::Pixmap,
}

impl<'a> Drop for Pixmap<'a> {
  fn drop(&mut self) {
    unsafe { xlib::XFreePixmap(self.display.inner, self.inner) };
  }
}

impl<'a> IntoInt for Pixmap<'a> {
  fn into_int(&self) -> u64 {
    self.inner
  }
}
